@@ -37,7 +37,7 @@ our @ISA = qw(Exporter);
 our @EXPORT = qw(shrink_epd expand_epd STR NAG);
 our @EXPORT_OK = qw();
 
-our $VERSION = '0.19'; # 10-jan-2006
+our $VERSION = '0.20'; # 23-Jan-2015
 
 =head1 NAME
 
@@ -232,7 +232,23 @@ sub new {
         GameComments =>{},    # comments with reference to the move
         gamedescr => {},    # will contain the PGN tags
         GameErrors => {},    # will contain the parsing errors
-        fh    => \$fh            # filehandle to the PGN file
+        fh    => \$fh,           # filehandle to the PGN file
+        # this is the memory between loops. The
+        # reading engine recognizes some elements
+        # one line after.
+        # For example, game text ends when we
+        # read tags from the input. At this moment,
+        # we have to return from the method, but
+        # we must keep in memory what we have last read.
+        # This structure will also take care of the 
+        # tags spanning over several lines.
+        memory => {
+            tag          => q{},
+            utag         => 0, # = unfinished tag
+            game         => q{},
+            tag_printed  => 0,
+            game_printed => 0,
+        }
     }, $class;
     return $self;
 }
@@ -693,78 +709,63 @@ games. The following deviance from the standard are handled:
 #
 # read_game() introduced in 0.07
 #
-{ #closure for read_game
-# this is the memory between loops. The
-# reading engine recognizes some elements
-# one line after.
-# For example, game text ends when we
-# read tags from the input. At this moment,
-# we have to return from the method, but
-# we must keep in memory what we have last read.
-# This structure will also take care of the 
-# tags spanning over several lines.
-my %memory = (    
-        tag          => q{},
-        utag         => 0, # = unfinished tag
-        game         => q{},
-        tag_printed  => 0,
-        game_printed => 0,
-    );
-
 sub _process_game {
      my $self = shift;
-     return 0 unless $memory{game};
-     $self->{gamedescr}{missing} .= 'tags' unless $memory{tag_printed};
-     $memory{tag_printed} = 0;
-     $self->{gamedescr}{Game} .= $memory{game}; 
-     $memory{game} = q{};
-     $memory{game_printed} =1;
+     my $memory = $self->{memory};
+     return 0 unless $memory->{game};
+     $self->{gamedescr}{missing} .= 'tags' unless $memory->{tag_printed};
+     $memory->{tag_printed} = 0;
+     $self->{gamedescr}{Game} .= $memory->{game}; 
+     $memory->{game} = q{};
+     $memory->{game_printed} =1;
      return 1;
 }
 
 sub _process_tag {
     my $self = shift;
-    if ($memory{game}) {
+    my $memory = $self->{memory};
+    if ($memory->{game}) {
         $self->_process_game;
     }
-    return 0 if $memory{utag};
-    if ($memory{tag} =~ tr/]// > 1) {
+    return 0 if $memory->{utag};
+    if ($memory->{tag} =~ tr/]// > 1) {
         # deals with multiple tags in one line
-        $memory{tag} =~ s/\]\s?/\]\n/g;
+        $memory->{tag} =~ s/\]\s?/\]\n/g;
     }
-    while ($memory{tag} =~ /\[(\w+)\s+"(.*)"\]\s*/g) {
+    while ($memory->{tag} =~ /\[(\w+)\s+"(.*)"\]\s*/g) {
         $self->{gamedescr}{$1} = $2;
     }
-    $memory{tag_printed} =1;
-    $memory{tag} = q{};
-    $memory{game_printed} = 0;
+    $memory->{tag_printed} =1;
+    $memory->{tag} = q{};
+    $memory->{game_printed} = 0;
     return;
 }
 
 sub read_game {
     my $self = shift;
     my $fh = ${$self->{fh}};
+    my $memory = $self->{memory};
     $self->_init();
-    $self->_process_tag if $memory{tag};
-    return $self->_process_game if $memory{game};
+    $self->_process_tag if $memory->{tag};
+    return $self->_process_game if $memory->{game};
     while (<$fh>) {
         # handle semicolon comments
         if (/^;/) {
-            if ($memory{game_printed} or (! $memory{game})) { # between games
+            if ($memory->{game_printed} or (! $memory->{game})) { # between games
                 chomp;
                 $self->{gamedescr}{Comment} .= $_ ;
                 # comments between games are saved as tags
             }
-            elsif ($memory{game}){
-                $memory{game} .= $_;
+            elsif ($memory->{game}){
+                $memory->{game} .= $_;
             }
             next; # anything else is discarded.
         }
         # normalize tagless games
         if (/^\s*$/) {
-            if ($memory{game}) {
+            if ($memory->{game}) {
                 # handles comments with embedded newlines.
-                if (($memory{game} =~ tr/\{//) < ($memory{game} =~ tr/\}//) ) {
+                if (($memory->{game} =~ tr/\{//) < ($memory->{game} =~ tr/\}//) ) {
                     next;
                 }
                 return $self->_process_game;
@@ -772,51 +773,50 @@ sub read_game {
             next;
         }
         # deals with multi-line tags
-        if ($memory{utag}) {
+        if ($memory->{utag}) {
             chomp;
-            $memory{tag} .= $_;
-            my $left_brackets = ($memory{tag} =~ tr/\[//);
-            my $right_brackets = ($memory{tag} =~ tr/\]//);
+            $memory->{tag} .= $_;
+            my $left_brackets = ($memory->{tag} =~ tr/\[//);
+            my $right_brackets = ($memory->{tag} =~ tr/\]//);
             if ( $left_brackets == $right_brackets ) {
-                $memory{utag}         = 0;
-                $memory{tag_printed}  = 0;
-                $memory{tag}        .= "\n";
+                $memory->{utag}         = 0;
+                $memory->{tag_printed}  = 0;
+                $memory->{tag}        .= "\n";
             }
         }
-        elsif (/^\[/ && (! $memory{game})) {
+        elsif (/^\[/ && (! $memory->{game})) {
             my $left_brackets = tr/\[//;
             my $right_brackets = tr/\]//;
             if ($left_brackets == $right_brackets) {
-                $memory{tag} = $_;
+                $memory->{tag} = $_;
             }
             elsif ($right_brackets > $left_brackets) {
                 warn "Parsing error at line $.\n";
             }
             else {
-                $memory{utag} = 1;
+                $memory->{utag} = 1;
                 chomp;
-                $memory{tag} = $_;
-                $memory{tag_printed} =0;
+                $memory->{tag} = $_;
+                $memory->{tag_printed} =0;
             }
         }
         else {
             s/^\s*//;
-            $memory{game} .= $_;
+            $memory->{game} .= $_;
         }
-        if ($memory{tag}) {
-            return $self->_process_game if $memory{game};
+        if ($memory->{tag}) {
+            return $self->_process_game if $memory->{game};
             $self->_process_tag;
         }
     }
-    if ($memory{tag}) {
+    if ($memory->{tag}) {
         $self->_process_tag;
     }
-    if ($memory{game}) {
+    if ($memory->{game}) {
         return $self->_process_game;
     }
     return 0;
 }
-} # end read_game() closure
 
 =item read_standard_game()
 
